@@ -7,20 +7,21 @@ import abc
 import hashlib
 import inspect
 import pathlib
+
+import importlib
+
 from dataclasses import dataclass
 from typing import List, Dict, Any, AsyncIterator, Type
 
-import numpy as np
 
 os.environ["TF_CPP_MIN_LOG_LEVEL"] = "2"
-import tensorflow as tf
 
 from dffml.record import Record
 from dffml.model.accuracy import Accuracy
 from dffml.base import config, field
-from dffml.source.source import Sources
 from dffml.util.entrypoint import entrypoint
 from dffml.feature.feature import Feature, Features
+from dffml.source.source import Sources, SourcesContext
 from dffml.model.model import ModelContext, Model, ModelNotTrained
 
 
@@ -28,14 +29,11 @@ from dffml.model.model import ModelContext, Model, ModelNotTrained
 class TensorflowBaseConfig:
     predict: Feature = field("Feature name holding target values")
     features: Features = field("Features to train on")
+    directory: pathlib.Path = field("Directory where state should be saved")
     steps: int = field("Number of steps to train the model", default=3000)
     epochs: int = field(
         "Number of iterations to pass over all records in a source", default=30
     )
-    directory: pathlib.Path = field(
-        "Directory where state should be saved",
-        default=pathlib.Path("~", ".cache", "dffml", "tensorflow"),
-    )
     hidden: List[int] = field(
         "List length is the number of hidden layers in the network. Each entry in the list is the number of nodes in that hidden layer",
         default_factory=lambda: [12, 40, 15],
@@ -51,6 +49,8 @@ class TensorflowModelContext(ModelContext):
 
     def __init__(self, parent):
         super().__init__(parent)
+        self.tf = importlib.import_module("tensorflow")
+        self.np = importlib.import_module("numpy")
         self._model = None
         self.feature_columns = self._feature_columns()
         self.features = self._applicable_features()
@@ -63,14 +63,14 @@ class TensorflowModelContext(ModelContext):
         for feature in self.parent.config.features:
             col = self._feature_feature_column(feature)
             if not col is None:
-                cols[feature.NAME] = col
+                cols[feature.name] = col
         return cols
 
     def _feature_feature_column(self, feature: Feature):
         """
         Creates a feature column for a feature
         """
-        dtype = feature.dtype()
+        dtype = feature.dtype
         if not inspect.isclass(dtype):
             self.logger.warning(
                 "Unknown dtype %r. Cound not create column" % (dtype)
@@ -82,8 +82,8 @@ class TensorflowModelContext(ModelContext):
             or dtype is float
             or issubclass(dtype, float)
         ):
-            return tf.feature_column.numeric_column(
-                feature.NAME, shape=feature.length()
+            return self.tf.feature_column.numeric_column(
+                feature.name, shape=feature.length
             )
         self.logger.warning(
             "Unknown dtype %r. Cound not create column" % (dtype)
@@ -105,31 +105,29 @@ class TensorflowModelContext(ModelContext):
         if self.parent.config.directory is None:
             return None
         _to_hash = self.features + list(map(str, self.parent.config.hidden))
-        model = hashlib.sha384("".join(_to_hash).encode("utf-8")).hexdigest()
+        model = "DNNModel"
         if not os.path.isdir(self.parent.config.directory):
             raise NotADirectoryError(
                 "%s is not a directory" % (self.parent.config.directory)
             )
         return os.path.join(self.parent.config.directory, model)
 
-    async def predict_input_fn(self, records: AsyncIterator[Record], **kwargs):
+    async def predict_input_fn(self, sources: SourcesContext, **kwargs):
         """
         Uses the numpy input function with data from record features.
         """
         x_cols: Dict[str, Any] = {feature: [] for feature in self.features}
         ret_records = []
-        async for record in records:
-            if not record.features(self.features):
-                continue
+        async for record in sources.with_features(self.features):
             ret_records.append(record)
             for feature, results in record.features(self.features).items():
-                x_cols[feature].append(np.array(results))
+                x_cols[feature].append(self.np.array(results))
         for feature in x_cols:
-            x_cols[feature] = np.array(x_cols[feature])
+            x_cols[feature] = self.np.array(x_cols[feature])
         self.logger.info("------ Record Data ------")
         self.logger.info("x_cols:    %d", len(list(x_cols.values())[0]))
         self.logger.info("-----------------------")
-        input_fn = tf.compat.v1.estimator.inputs.numpy_input_fn(
+        input_fn = self.tf.compat.v1.estimator.inputs.numpy_input_fn(
             x_cols, shuffle=False, num_epochs=1, **kwargs
         )
         return input_fn, ret_records
@@ -141,14 +139,14 @@ class TensorflowModelContext(ModelContext):
         input_fn = await self.training_input_fn(sources)
         self.model.train(input_fn=input_fn, steps=self.parent.config.steps)
 
-    async def get_predictions(self, records: Record):
+    async def get_predictions(self, sources: SourcesContext):
         if not os.path.isdir(self.model_dir_path):
             raise ModelNotTrained("Train model before prediction.")
         # Create the input function
-        input_fn, predict = await self.predict_input_fn(records)
+        input_fn, predict = await self.predict_input_fn(sources)
         # Makes predictions on classifications
         predictions = self.model.predict(input_fn=input_fn)
-        target = self.parent.config.predict.NAME
+        target = self.parent.config.predict.name
 
         return predict, predictions, target
 
@@ -222,7 +220,7 @@ class DNNClassifierModelContext(TensorflowModelContext):
             len(self.classifications),
             self.classifications,
         )
-        self._model = tf.compat.v1.estimator.DNNClassifier(
+        self._model = self.tf.compat.v1.estimator.DNNClassifier(
             feature_columns=list(self.feature_columns.values()),
             hidden_units=self.parent.config.hidden,
             n_classes=len(self.parent.config.classifications),
@@ -236,23 +234,27 @@ class DNNClassifierModelContext(TensorflowModelContext):
         for record in [
             record
             async for record in sources.with_features(
-                self.features + [self.parent.config.predict.NAME]
+                self.features + [self.parent.config.predict.name]
+            )
+            if self.parent.config.clstype(
+                record.feature(self.parent.config.predict.name)
             )
-            if record.feature(self.parent.config.predict.NAME)
             in self.classifications
         ]:
             for feature, results in record.features(self.features).items():
-                x_cols[feature].append(np.array(results))
+                x_cols[feature].append(self.np.array(results))
             y_cols.append(
                 self.classifications[
-                    record.feature(self.parent.config.predict.NAME)
+                    self.parent.config.clstype(
+                        record.feature(self.parent.config.predict.name)
+                    )
                 ]
             )
         if not y_cols:
             raise ValueError("No records to train on")
-        y_cols = np.array(y_cols)
+        y_cols = self.np.array(y_cols)
         for feature in x_cols:
-            x_cols[feature] = np.array(x_cols[feature])
+            x_cols[feature] = self.np.array(x_cols[feature])
 
         return x_cols, y_cols
 
@@ -266,7 +268,7 @@ class DNNClassifierModelContext(TensorflowModelContext):
         self.logger.info("x_cols:    %d", len(list(x_cols.values())[0]))
         self.logger.info("y_cols:    %d", len(y_cols))
         self.logger.info("-----------------------")
-        input_fn = tf.compat.v1.estimator.inputs.numpy_input_fn(
+        input_fn = self.tf.compat.v1.estimator.inputs.numpy_input_fn(
             x_cols,
             y_cols,
             batch_size=self.parent.config.batchsize,
@@ -285,7 +287,7 @@ class DNNClassifierModelContext(TensorflowModelContext):
         self.logger.info("x_cols:    %d", len(list(x_cols.values())[0]))
         self.logger.info("y_cols:    %d", len(y_cols))
         self.logger.info("-----------------------")
-        input_fn = tf.compat.v1.estimator.inputs.numpy_input_fn(
+        input_fn = self.tf.compat.v1.estimator.inputs.numpy_input_fn(
             x_cols,
             y_cols,
             batch_size=self.parent.config.batchsize,
@@ -306,13 +308,11 @@ class DNNClassifierModelContext(TensorflowModelContext):
         accuracy_score = self.model.evaluate(input_fn=input_fn)
         return Accuracy(accuracy_score["accuracy"])
 
-    async def predict(
-        self, records: AsyncIterator[Record]
-    ) -> AsyncIterator[Record]:
+    async def predict(self, sources: SourcesContext) -> AsyncIterator[Record]:
         """
         Uses trained data to make a prediction about the quality of a record.
         """
-        predict, predictions, target = await self.get_predictions(records)
+        predict, predictions, target = await self.get_predictions(sources)
         for record, pred_dict in zip(predict, predictions):
             class_id = pred_dict["class_ids"][0]
             probability = pred_dict["probabilities"][class_id]
