@@ -3,19 +3,17 @@ Uses Tensorflow to create a generic DNN which learns on all of the features in a
 record.
 """
 import os
+import importlib
 from typing import Dict, Any, AsyncIterator
 
-import numpy as np
-
 os.environ["TF_CPP_MIN_LOG_LEVEL"] = "2"
-import tensorflow as tf
 
 from dffml.base import config
 from dffml.record import Record
 from dffml.model.model import Model
 from dffml.model.accuracy import Accuracy
-from dffml.source.source import Sources
 from dffml.util.entrypoint import entrypoint
+from dffml.source.source import Sources, SourcesContext
 
 from .dnnc import TensorflowModelContext, TensorflowBaseConfig
 
@@ -33,9 +31,11 @@ class DNNRegressionModelContext(TensorflowModelContext):
 
     def __init__(self, parent) -> None:
         super().__init__(parent)
+        self.tf = importlib.import_module("tensorflow")
+        self.np = importlib.import_module("numpy")
         self.model_dir_path = self._model_dir_path()
         self.all_features = self.parent.config.features.names() + [
-            self.parent.config.predict.NAME
+            self.parent.config.predict.name
         ]
         self.features = self._applicable_features()
 
@@ -48,7 +48,7 @@ class DNNRegressionModelContext(TensorflowModelContext):
             return self._model
         self.logger.debug("Loading model ")
 
-        self._model = tf.compat.v1.estimator.DNNRegressor(
+        self._model = self.tf.compat.v1.estimator.DNNRegressor(
             feature_columns=list(self.feature_columns.values()),
             hidden_units=self.parent.config.hidden,
             model_dir=self.model_dir_path,
@@ -63,12 +63,12 @@ class DNNRegressionModelContext(TensorflowModelContext):
         async for record in sources.with_features(self.all_features):
             for feature, results in record.features(self.features).items():
 
-                x_cols[feature].append(np.array(results))
-            y_cols.append(record.feature(self.parent.config.predict.NAME))
+                x_cols[feature].append(self.np.array(results))
+            y_cols.append(record.feature(self.parent.config.predict.name))
 
-        y_cols = np.array(y_cols)
+        y_cols = self.np.array(y_cols)
         for feature in x_cols:
-            x_cols[feature] = np.array(x_cols[feature])
+            x_cols[feature] = self.np.array(x_cols[feature])
 
         return x_cols, y_cols
 
@@ -89,7 +89,7 @@ class DNNRegressionModelContext(TensorflowModelContext):
         self.logger.info("x_cols:    %d", len(list(x_cols.values())[0]))
         self.logger.info("y_cols:    %d", len(y_cols))
         self.logger.info("-----------------------")
-        input_fn = tf.compat.v1.estimator.inputs.numpy_input_fn(
+        input_fn = self.tf.compat.v1.estimator.inputs.numpy_input_fn(
             x_cols,
             y_cols,
             batch_size=batch_size,
@@ -115,7 +115,7 @@ class DNNRegressionModelContext(TensorflowModelContext):
         self.logger.info("x_cols:    %d", len(list(x_cols.values())[0]))
         self.logger.info("y_cols:    %d", len(y_cols))
         self.logger.info("-----------------------")
-        input_fn = tf.compat.v1.estimator.inputs.numpy_input_fn(
+        input_fn = self.tf.compat.v1.estimator.inputs.numpy_input_fn(
             x_cols,
             y_cols,
             batch_size=batch_size,
@@ -138,13 +138,11 @@ class DNNRegressionModelContext(TensorflowModelContext):
         metrics = self.model.evaluate(input_fn=input_fn)
         return Accuracy(1 - metrics["loss"])  # 1 - mse
 
-    async def predict(
-        self, records: AsyncIterator[Record]
-    ) -> AsyncIterator[Record]:
+    async def predict(self, sources: SourcesContext) -> AsyncIterator[Record]:
         """
         Uses trained data to make a prediction about the quality of a record.
         """
-        predict, predictions, target = await self.get_predictions(records)
+        predict, predictions, target = await self.get_predictions(sources)
         for record, pred_dict in zip(predict, predictions):
             # TODO Instead of float("nan") save accuracy value and use that.
             record.predicted(
@@ -205,7 +203,7 @@ class DNNRegressionModel(Model):
                 "last_updated": "2019-10-24T15:26:41Z",
                 "prediction": {
                     "TARGET" : {
-                        "confidence": NaN,
+                        "confidence": null,
                         "value": 1.1983429193496704
                     }
                 },
@@ -217,7 +215,7 @@ class DNNRegressionModel(Model):
 
     .. literalinclude:: /../model/tensorflow/examples/tfdnnr/tfdnnr.py
 
-    The ``NaN`` in ``confidence`` is the expected behaviour. (See TODO in
+    The ``null`` in ``confidence`` is the expected behaviour. (See TODO in
     predict).
 
     """
